//
//      Block.hpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.0 (English Edition)
//

#ifndef __BLOCK_HPP
#define __BLOCK_HPP

#include <SophiaFramework.hpp>
#include "Block.bid"
#include "Parameters.hpp"
#include "Overlap.hpp"

// Structure for saving ball properties
SFMTYPEDEFSTRUCT(BallData)
struct BallData {
    SFXCircle ball;                             // Ball
    SFXRGBColor color;                          // Color
    SFXGrid velocity;                           // Speed
};

// Structure for storing racket properties
SFMTYPEDEFSTRUCT(RacketData)
struct RacketData {
    SFXRectangle racket;                        // Racket
    SFXRGBColor color;                          // Color
    SFXGrid velocity;                           // Speed
};

// Structure for storing block properties
SFMTYPEDEFSTRUCT(BlockData)
struct BlockData {
    UInt16 type;                                // Block type
    SInt16 life;                                // Block life (how many balls can it stand)
    SInt16 score;                               // Score
    SFXRectangle block;                         // Block
    SFXRGBColor color;                          // Color
};


SFMTYPEDEFCLASS(Block)

class Block : public SFCApplication {

    SFMSEALCOPY(Block)

public:

    static SFCInvokerPtr Factory(Void);
    Void MoveBlocks(Void);                  // The entire block is moved. (Entity)
    Void Animate(Void);                     // Collision detection and animation(Entity)

private:

    Block(Void);
    virtual ~Block(Void);
    virtual Bool Invoke(SFXEventConstRef event);
    Void OnAppStart(AEEAppStartPtr environment);
    Void OnAppResume(AEEAppStartPtr environment);
    Void OnAppSuspend(AEESuspendReason reason, AEESuspendInfoPtr info);
    Bool OnKey(UInt16 key);

    // Variables for Breakout
    SFXGraphicsPtr _pgraphics;              // Graphics interface
    SFXRectangle _gameArea;                 // Game Area
    SFXRectangle _messageArea;              // Area for message display
    SFXRectangle _basicBlock;               // Standard block, other blocks copied from this one
    BallData _usingBall;                    // Ball used to play
    Bool _ballPassing;                      // Is the ball passing through the block?
    BallData _spareBalls[SPARE_BALLS];      // Spare balls
    RacketData _racket;                     // Racket
    BlockData _blocks[BLOCK_H_NUM * BLOCK_V_NUM];   // Block
    SInt32 _score;                          // Score
    SInt16 _spareBallNumber;                // Number of spare balls
    SInt16 _blockNumber;                    // Number of blocks remaining
    Bool _blockMotion;                      // Stage where block moves?
    SInt16 _nowStage;                       // Current stage
    SInt32 _drawInterval;                   // Redrawing interval (unit millisconds)
    SInt16 _ballYSpeed;                     // Ball speed in y direction (how many dots per frame)
    SInt16 _racketSpeed;                    // Racket speed (how many dots per frame)
    UInt16 _status;                         // Current application status

    // Functions only for Breakout

    Void Initialize(Void);                  // Basic initialization function
    Void DrawBlock(SInt16 n = -1, Bool flag = true);    // Draw nth block  (if flag = false, delete it)
    Void DrawRacket(Bool flag = true);      // Draw Raquet  (if flag = false, delete it)
    Void DrawBall(Bool flag = true);        // Draw ball  (if flag = false, delete it)
    Void DrawMessage(SFXWideStringConstRef msg);    // Display message (message, number of spare balls)
    Void DrawMessage(Void);                 // Display of message(score and spare ball number)
    Void CreateStage(SInt16 n);             // Generate stage n
    UInt16 CheckEnd(Void);                  // Finished stage?
    Void ShowGameOver(Void);                // Game over
    Void ShowGameClear(Void);               // Game clear!
    Void Continue(Void);                    // Continue?
    Void Options(Void);                     // Option menu
    Void Pause(Void);                       // Pauses the game

};

Void CBMoveBlocks(VoidPtr pdata);           // Move whole block (for timer registration)
Void CBAnimate(VoidPtr pdata);              // Collision detection and animation (for timer registration)

#endif // __BLOCK_HPP //
